"""JARVIS ban tasks."""
import asyncio
from datetime import datetime, timedelta
from logging import Logger

from dis_snek import Snake
from dis_snek.client.errors import NotFound
from jarvis_core.db import q
from jarvis_core.db.models import Ban, Unban


async def unban(bot: Snake, logger: Logger) -> None:
    """
    Unban users when ban expires.

    Args:
        bot: Snake instance
        logger: Global logger
    """
    while True:
        max_time = datetime.utcnow() + timedelta(minutes=10)
        bans = Ban.find(q(type="temp", active=True))
        async for ban in bans:
            if ban.created_at + timedelta(hours=ban.duration) < max_time:
                guild = await bot.fetch_guild(ban.guild)
                user = await bot.fetch_user(ban.user)
                if guild and user:
                    logger.debug(f"Unbanned user {user.id} from guild {guild.id}")
                    try:
                        await guild.unban(user=user, reason="JARVIS tempban expired")
                    except NotFound:
                        logger.debug(f"User {user.id} not banned from guild {guild.id}")

                ban.update(q(active=False))
                await ban.commit()
                u = Unban(
                    user=user.id,
                    guild=guild.id,
                    username=user.username,
                    discrim=user.discriminator,
                    admin=bot.user.id,
                    reason="Ban expired",
                )
                await u.commit()

        # Check ever 10 minutes
        await asyncio.sleep(600)
